@@ -1879,6 +1879,7 @@ end = struct
     let { Info.scope; hook } = pinfo.Proof_info.info in
     List.map_i (
       fun i { CInfo.name; typ; impargs } ->
+        let typ = Evarutil.nf_evars_universes (Evd.from_ctx uctx) typ in
         let pe = (sec_vars, (typ, univs), None) in
         declare_assumption ~name ~scope ~hook ~impargs ~uctx pe
     ) 0 pinfo.Proof_info.cinfo
@@ -2100,9 +2101,7 @@ module Error = struct
     CErrors.user_err (Pp.str (Printf.sprintf "Unknown obligation number %i" (succ num)))
 
   let already_solved num =
-    CErrors.user_err
-      ( str "Obligation" ++ spc () ++ int num ++ str "already" ++ spc ()
-        ++ str "solved." )
+    CErrors.user_err Pp.(str "Obligation " ++ int num ++ str " already solved." )
 
   let depends num rem =
     CErrors.user_err
@@ -2307,7 +2306,7 @@ let obligation (user_num, name, typ) ~pm tac =
     let obl = obls.(num) in
     match obl.obl_body with
     | None -> solve_obligation prg num tac
-    | Some r -> Error.already_solved num
+    | Some r -> Error.already_solved user_num
   else Error.unknown_obligation num
 
 let show_single_obligation i n obls x =
